@@ -1,3 +1,4 @@
+import logging
 import os
 from contextlib import contextmanager
 from typing import Generator, Optional, Tuple, Union
@@ -5,7 +6,7 @@ from typing import Generator, Optional, Tuple, Union
 import docker
 import requests
 from requests.adapters import HTTPAdapter
-from requests.packages.urllib3.util.retry import Retry
+from urllib3.util.retry import Retry
 
 from .config import Config
 from .data_model import *
@@ -40,6 +41,10 @@ class Beaker:
             around indefinitely, consider using the :meth:`session()` context manager
             intermittently instead.
 
+    :param pool_maxsize: The maximum size of the connection pool to use.
+        If not specified, a large default value will be used based on a multiple of the number
+        of CPUs available.
+
     The easiest way to initialize a Beaker client is with :meth:`.from_env()`:
 
     >>> beaker = Beaker.from_env()
@@ -57,21 +62,24 @@ class Beaker:
 
     RECOVERABLE_SERVER_ERROR_CODES = (429, 500, 502, 503, 504)
     MAX_RETRIES = 5
+    BACKOFF_FACTOR = 1
+    BACKOFF_MAX = 120
+
     API_VERSION = "v3"
 
+    logger = logging.getLogger("beaker")
+
     def __init__(
         self,
         config: Config,
         check_for_upgrades: bool = True,
         timeout: Optional[Union[float, Tuple[float, float]]] = 5.0,
         session: Optional[Union[bool, requests.Session]] = None,
+        pool_maxsize: Optional[int] = None,
     ):
-        # See if there's a newer version, and if so, suggest that the user upgrades.
-        if check_for_upgrades:
-            self._check_for_upgrades()
-
         self._config = config
         self._docker: Optional[docker.DockerClient] = None
+        self._pool_maxsize = pool_maxsize or min(100, (os.cpu_count() or 16) * 6)
         self._session: Optional[requests.Session] = (
             None
             if not session
@@ -104,6 +112,10 @@ class Beaker:
                 raise ValueError("'default_org' cannot be an empty string")
             self.organization.get(self._config.default_org)
 
+        # See if there's a newer version, and if so, suggest that the user upgrades.
+        if check_for_upgrades:
+            self._check_for_upgrades()
+
     def __str__(self) -> str:
         return (
             f"Beaker("
@@ -144,7 +156,7 @@ class Beaker:
                         f"https://github.com/allenai/beaker-py/releases/tag/v{latest_version}\n",
                         UserWarning,
                     )
-        except (requests.exceptions.Timeout, requests.exceptions.ConnectionError):
+        except Exception:
             pass
 
     @classmethod
@@ -153,6 +165,7 @@ class Beaker:
         check_for_upgrades: bool = True,
         timeout: Optional[Union[float, Tuple[float, float]]] = 5.0,
         session: Optional[Union[bool, requests.Session]] = None,
+        pool_maxsize: Optional[int] = None,
         **overrides,
     ) -> "Beaker":
         """
@@ -177,6 +190,10 @@ class Beaker:
                 around indefinitely, consider using the :meth:`session()` context manager
                 intermittently instead.
 
+        :param pool_maxsize: The maximum size of the connection pool to use.
+            If not specified, a large default value will be used based on a multiple of the number
+            of CPUs available.
+
         :param overrides: Fields in the :class:`Config` to override.
 
         .. note::
@@ -192,6 +209,7 @@ class Beaker:
             check_for_upgrades=check_for_upgrades,
             timeout=timeout,
             session=session,
+            pool_maxsize=pool_maxsize,
         )
 
     def _make_session(self) -> requests.Session:
@@ -199,15 +217,11 @@ class Beaker:
         retries = Retry(
             total=self.MAX_RETRIES * 2,
             connect=self.MAX_RETRIES,
-            read=self.MAX_RETRIES,
             status=self.MAX_RETRIES,
-            other=self.MAX_RETRIES,
-            backoff_factor=1,
+            backoff_factor=self.BACKOFF_FACTOR,
             status_forcelist=self.RECOVERABLE_SERVER_ERROR_CODES,
         )
-        session.mount(
-            "https://", HTTPAdapter(max_retries=retries, pool_maxsize=(os.cpu_count() or 16) * 2)
-        )
+        session.mount("https://", HTTPAdapter(max_retries=retries, pool_maxsize=self._pool_maxsize))
         return session
 
     @contextmanager
