@@ -6,7 +6,23 @@ from pydantic import Field, validator
 
 from .account import Account
 from .base import BaseModel
-from .experiment_spec import Priority, TaskSpec
+from .experiment_spec import DataMount, EnvVar, ImageSource, Priority, TaskSpec
+
+__all__ = [
+    "CurrentJobStatus",
+    "CanceledCode",
+    "JobStatus",
+    "ExecutionResult",
+    "JobRequests",
+    "JobLimits",
+    "JobExecution",
+    "JobKind",
+    "Job",
+    "Jobs",
+    "JobStatusUpdate",
+    "JobPatch",
+    "Session",
+]
 
 
 class CurrentJobStatus(str, Enum):
@@ -22,6 +38,14 @@ class CurrentJobStatus(str, Enum):
     failed = "failed"
     finalized = "finalized"
     canceled = "canceled"
+    preempted = "preempted"
+
+
+class CanceledCode(int, Enum):
+    not_set = 0
+    system_preemption = 1
+    user_preemption = 2
+    idle = 3
 
 
 class JobStatus(BaseModel):
@@ -32,6 +56,8 @@ class JobStatus(BaseModel):
     failed: Optional[datetime] = None
     finalized: Optional[datetime] = None
     canceled: Optional[datetime] = None
+    canceled_for: Optional[str] = None
+    canceled_code: Optional[CanceledCode] = None
     idle_since: Optional[datetime] = None
     exit_code: Optional[int] = None
     message: Optional[str] = None
@@ -58,7 +84,10 @@ class JobStatus(BaseModel):
         elif self.exited is not None:
             return CurrentJobStatus.exited
         elif self.canceled is not None:
-            return CurrentJobStatus.canceled
+            if self.canceled_code in {CanceledCode.system_preemption, CanceledCode.user_preemption}:
+                return CurrentJobStatus.preempted
+            else:
+                return CurrentJobStatus.canceled
         elif self.idle_since is not None:
             return CurrentJobStatus.idle
         elif self.started is not None:
@@ -93,6 +122,7 @@ class JobExecution(BaseModel):
     experiment: str
     spec: TaskSpec
     result: ExecutionResult
+    workspace: Optional[str] = None
 
 
 class JobKind(str, Enum):
@@ -104,6 +134,17 @@ class JobKind(str, Enum):
     session = "session"
 
 
+class Session(BaseModel):
+    command: Optional[Tuple[str, ...]] = None
+    env_vars: Optional[Tuple[EnvVar, ...]] = None
+    datasets: Optional[Tuple[DataMount, ...]] = None
+    image: Optional[ImageSource] = None
+    save_image: bool = False
+    ports: Optional[Tuple[int, ...]] = None
+    priority: Optional[Priority] = None
+    work_dir: Optional[str] = None
+
+
 class Job(BaseModel):
     """
     A :class:`Job` is an execution of a :class:`Task`.
@@ -123,6 +164,7 @@ class Job(BaseModel):
     node: Optional[str] = None
     requests: Optional[JobRequests] = None
     limits: Optional[JobLimits] = None
+    session: Optional[Session] = None
 
     @property
     def display_name(self) -> str:
@@ -134,6 +176,9 @@ class Job(BaseModel):
 
     @property
     def is_done(self) -> bool:
+        """
+        Same as :meth:`is_finalized()`, kept for backwards compatibility.
+        """
         return self.status.current == CurrentJobStatus.finalized
 
     def check(self):
@@ -157,13 +202,6 @@ class Jobs(BaseModel):
     next: Optional[str] = None
 
 
-class CanceledCode(int, Enum):
-    not_set = 0
-    system_preemption = 1
-    user_preemption = 2
-    idle = 3
-
-
 class JobStatusUpdate(BaseModel):
     scheduled: Optional[bool] = None
     started: Optional[bool] = None
