@@ -1,3 +1,4 @@
+import io
 import os
 from datetime import datetime
 from pathlib import Path
@@ -6,10 +7,11 @@ from typing import TYPE_CHECKING, Deque, Dict, Generator, Optional, Tuple, Union
 from ..aliases import PathOrStr
 from ..data_model import *
 from ..exceptions import *
-from ..util import path_is_relative_to
+from ..util import log_and_wait, path_is_relative_to, retriable
 from .service_client import ServiceClient
 
 if TYPE_CHECKING:
+    from requests import Response
     from rich.progress import Progress, TaskID
 
 
@@ -34,7 +36,8 @@ class DatasetClient(ServiceClient):
         :param dataset: The dataset ID or name.
 
         :raises DatasetNotFound: If the dataset can't be found.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
 
         """
 
@@ -102,7 +105,8 @@ class DatasetClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         self.validate_beaker_name(name)
         workspace_id = self.resolve_workspace(workspace).id
@@ -157,17 +161,25 @@ class DatasetClient(ServiceClient):
 
         :raises DatasetNotFound: If the dataset can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         dataset_id = self.resolve_dataset(dataset).id
-        return Dataset.from_json(
-            self.request(
-                f"datasets/{self.url_quote(dataset_id)}",
-                method="PATCH",
-                data=DatasetPatch(commit=True),
-                exceptions_for_status={404: DatasetNotFound(self._not_found_err_msg(dataset))},
-            ).json()
-        )
+
+        @retriable()
+        def commit() -> Dataset:
+            # It's okay to retry this because committing a dataset multiple
+            # times does nothing.
+            return Dataset.from_json(
+                self.request(
+                    f"datasets/{self.url_quote(dataset_id)}",
+                    method="PATCH",
+                    data=DatasetPatch(commit=True),
+                    exceptions_for_status={404: DatasetNotFound(self._not_found_err_msg(dataset))},
+                ).json()
+            )
+
+        return commit()
 
     def fetch(
         self,
@@ -195,7 +207,8 @@ class DatasetClient(ServiceClient):
         :raises ChecksumFailedError: If ``validate_checksum=True`` and the digest of one of the
             downloaded files doesn't match the expected digest.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         dataset = self.resolve_dataset(dataset)
         if dataset.storage is None:
@@ -268,20 +281,21 @@ class DatasetClient(ServiceClient):
         file: Union[str, FileInfo],
         offset: int = 0,
         length: int = -1,
-        max_retries: int = 5,
         quiet: bool = False,
         validate_checksum: bool = True,
     ) -> Generator[bytes, None, None]:
         """
         Stream download the contents of a single file from a dataset.
 
+        .. seealso::
+            :meth:`get_file()` is similar but returns the entire contents at once instead of
+            a generator over the contents.
+
         :param dataset: The dataset ID, name, or object.
         :param file: The path of the file within the dataset or the corresponding
             :class:`~beaker.data_model.dataset.FileInfo` object.
         :param offset: Offset to start from, in bytes.
         :param length: Number of bytes to read.
-        :param max_retries: Number of times to restart the download when HTTP errors occur.
-            Errors can be expected for very large files.
         :param quiet: If ``True``, progress won't be displayed.
         :param validate_checksum: If ``True``, the checksum of the downloaded bytes will be verified.
 
@@ -291,7 +305,8 @@ class DatasetClient(ServiceClient):
         :raises ChecksumFailedError: If ``validate_checksum=True`` and the digest of the downloaded
             bytes don't match the expected digest.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
 
         :examples:
 
@@ -315,12 +330,64 @@ class DatasetClient(ServiceClient):
                 file_info,
                 offset=offset,
                 length=length,
-                max_retries=max_retries,
                 validate_checksum=validate_checksum,
             ):
                 progress.update(task_id, advance=len(bytes_chunk))
                 yield bytes_chunk
 
+    def get_file(
+        self,
+        dataset: Union[str, Dataset],
+        file: Union[str, FileInfo],
+        offset: int = 0,
+        length: int = -1,
+        quiet: bool = False,
+        validate_checksum: bool = True,
+    ) -> bytes:
+        """
+        Download the contents of a single file from a dataset.
+
+        .. seealso::
+            :meth:`stream_file()` is similar but returns a generator over the contents.
+
+        :param dataset: The dataset ID, name, or object.
+        :param file: The path of the file within the dataset or the corresponding
+            :class:`~beaker.data_model.dataset.FileInfo` object.
+        :param offset: Offset to start from, in bytes.
+        :param length: Number of bytes to read.
+        :param quiet: If ``True``, progress won't be displayed.
+        :param validate_checksum: If ``True``, the checksum of the downloaded bytes will be verified.
+
+        :raises DatasetNotFound: If the dataset can't be found.
+        :raises DatasetReadError: If the :data:`~beaker.data_model.dataset.Dataset.storage` hasn't been set.
+        :raises FileNotFoundError: If the file doesn't exist in the dataset.
+        :raises ChecksumFailedError: If ``validate_checksum=True`` and the digest of the downloaded
+            bytes don't match the expected digest.
+        :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
+
+        :examples:
+
+        >>> contents = beaker.dataset.get_file(squad_dataset_name, "squad-train.arrow", quiet=True)
+        <BLANKLINE>
+        """
+
+        @retriable(recoverable_errors=(RequestException, ChecksumFailedError))
+        def _get_file() -> bytes:
+            return b"".join(
+                self.stream_file(
+                    dataset,
+                    file,
+                    offset=offset,
+                    length=length,
+                    quiet=quiet,
+                    validate_checksum=validate_checksum,
+                )
+            )
+
+        return _get_file()
+
     def file_info(self, dataset: Union[str, Dataset], file_name: str) -> FileInfo:
         """
         Get the :class:`~beaker.data_model.dataset.FileInfo` for a file in a dataset.
@@ -332,7 +399,8 @@ class DatasetClient(ServiceClient):
         :raises DatasetReadError: If the :data:`~beaker.data_model.dataset.Dataset.storage` hasn't been set.
         :raises FileNotFoundError: If the file doesn't exist in the dataset.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         dataset = self.resolve_dataset(dataset, ensure_storage=True)
         assert dataset.storage is not None
@@ -362,7 +430,8 @@ class DatasetClient(ServiceClient):
 
         :raises DatasetNotFound: If the dataset can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         dataset_id = self.resolve_dataset(dataset).id
         self.request(
@@ -404,7 +473,8 @@ class DatasetClient(ServiceClient):
         :raises UnexpectedEOFError: If a source is an empty file, or if a source is a directory and
             the contents of one of the directory's files changes while creating the dataset.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         dataset = self.resolve_dataset(dataset)
         if dataset.committed is not None:
@@ -417,13 +487,13 @@ class DatasetClient(ServiceClient):
             total_bytes = 0
             # map source path to (target_path, size)
             path_info: Dict[Path, Tuple[Path, int]] = {}
-            for name in sources:
-                source = Path(name)
+            for source in sources:
+                source = Path(source)
                 strip_path = strip_paths or not path_is_relative_to(source, ".")
                 if source.is_file():
                     target_path = Path(source.name) if strip_path else source
                     if target is not None:
-                        target_path = Path(target) / target_path
+                        target_path = Path(str(target)) / target_path
                     size = source.lstat().st_size
                     if size == 0:
                         raise UnexpectedEOFError(f"'{source}', empty files are not allowed")
@@ -435,7 +505,7 @@ class DatasetClient(ServiceClient):
                             continue
                         target_path = path.relative_to(source) if strip_path else path
                         if target is not None:
-                            target_path = Path(target) / target_path
+                            target_path = Path(str(target)) / target_path
                         size = path.lstat().st_size
                         if size == 0:
                             continue
@@ -475,6 +545,40 @@ class DatasetClient(ServiceClient):
                         total_bytes += actual_size - original_size
                         progress.update(bytes_task, total=total_bytes)
 
+    def upload(
+        self,
+        dataset: Union[str, Dataset],
+        source: bytes,
+        target: PathOrStr,
+        quiet: bool = False,
+    ) -> None:
+        """
+        Upload raw bytes to an uncommitted dataset.
+
+        :param dataset: The dataset ID, name, or object.
+        :param source: The raw bytes to upload to the dataset.
+        :param target: The name to assign to the file for the bytes in the dataset.
+        :param quiet: If ``True``, progress won't be displayed.
+
+        :raises DatasetNotFound: If the dataset can't be found.
+        :raises DatasetWriteError: If the dataset was already committed.
+        :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
+        """
+        dataset = self.resolve_dataset(dataset)
+        if dataset.committed is not None:
+            raise DatasetWriteError(dataset.id)
+
+        from ..progress import get_dataset_sync_progress
+
+        size = len(source)
+        with get_dataset_sync_progress(quiet) as progress:
+            task_id = progress.add_task("Uploading source")
+            if size is not None:
+                progress.update(task_id, total=size)
+            self._upload_file(dataset, size, source, target, progress, task_id)
+
     def ls(self, dataset: Union[str, Dataset]) -> Generator[FileInfo, None, None]:
         """
         List files in a dataset.
@@ -484,7 +588,8 @@ class DatasetClient(ServiceClient):
         :raises DatasetNotFound: If the dataset can't be found.
         :raises DatasetReadError: If the :data:`~beaker.data_model.dataset.Dataset.storage` hasn't been set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         dataset = self.resolve_dataset(dataset)
         if dataset.storage is None:
@@ -503,7 +608,8 @@ class DatasetClient(ServiceClient):
 
         :raises DatasetNotFound: If the dataset can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         total = 0
         for file_info in self.ls(dataset):
@@ -522,7 +628,8 @@ class DatasetClient(ServiceClient):
         :raises DatasetNotFound: If the dataset can't be found.
         :raises DatasetConflict: If a dataset by that name already exists.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         self.validate_beaker_name(name)
         dataset_id = self.resolve_dataset(dataset).id
@@ -560,49 +667,68 @@ class DatasetClient(ServiceClient):
         self,
         dataset: Dataset,
         size: int,
-        source: PathOrStr,
+        source: Union[PathOrStr, bytes],
         target: PathOrStr,
         progress: "Progress",
         task_id: "TaskID",
         ignore_errors: bool = False,
     ) -> int:
-        source = Path(source)
+        from ..progress import BufferedReaderWithProgress
+
         assert dataset.storage is not None
-        if ignore_errors and not source.exists():
-            return 0
 
-        with source.open("rb") as source_file:
-            from ..progress import BufferedReaderWithProgress
+        source_file_wrapper: BufferedReaderWithProgress
+        if isinstance(source, (str, Path, os.PathLike)):
+            source = Path(source)
+            if ignore_errors and not source.exists():
+                return 0
+            source_file_wrapper = BufferedReaderWithProgress(source.open("rb"), progress, task_id)
+        elif isinstance(source, bytes):
+            source_file_wrapper = BufferedReaderWithProgress(io.BytesIO(source), progress, task_id)
+        else:
+            raise ValueError(f"Expected path-like or raw bytes, got {type(source)}")
 
-            source_file_wrapper = BufferedReaderWithProgress(source_file, progress, task_id)
+        try:
             body: Optional[BufferedReaderWithProgress] = source_file_wrapper
             digest: Optional[str] = None
 
             if size > self.REQUEST_SIZE_LIMIT:
-                response = self.request(
-                    "uploads",
-                    method="POST",
-                    token=dataset.storage.token,
-                    base_url=dataset.storage.address,
-                )
-                upload_id = response.headers[self.HEADER_UPLOAD_ID]
+
+                @retriable()
+                def get_upload_id() -> str:
+                    assert dataset.storage is not None  # for mypy
+                    response = self.request(
+                        "uploads",
+                        method="POST",
+                        token=dataset.storage.token,
+                        base_url=dataset.storage.address,
+                    )
+                    return response.headers[self.HEADER_UPLOAD_ID]
+
+                upload_id = get_upload_id()
 
                 written = 0
                 while written < size:
                     chunk = source_file_wrapper.read(self.REQUEST_SIZE_LIMIT)
                     if not chunk:
                         break
-                    response = self.request(
-                        f"uploads/{upload_id}",
-                        method="PATCH",
-                        data=chunk,
-                        token=dataset.storage.token,
-                        base_url=dataset.storage.address,
-                        headers={
-                            self.HEADER_UPLOAD_LENGTH: str(size),
-                            self.HEADER_UPLOAD_OFFSET: str(written),
-                        },
-                    )
+
+                    @retriable()
+                    def upload() -> "Response":
+                        assert dataset.storage is not None  # for mypy
+                        return self.request(
+                            f"uploads/{upload_id}",
+                            method="PATCH",
+                            data=chunk,
+                            token=dataset.storage.token,
+                            base_url=dataset.storage.address,
+                            headers={
+                                self.HEADER_UPLOAD_LENGTH: str(size),
+                                self.HEADER_UPLOAD_OFFSET: str(written),
+                            },
+                        )
+
+                    response = upload()
                     written += len(chunk)
 
                     digest = response.headers.get(self.HEADER_DIGEST)
@@ -614,21 +740,28 @@ class DatasetClient(ServiceClient):
 
                 body = None
 
-            self.request(
-                f"datasets/{dataset.storage.id}/files/{str(target)}",
-                method="PUT",
-                data=body,
-                token=dataset.storage.token,
-                base_url=dataset.storage.address,
-                headers=None if not digest else {self.HEADER_DIGEST: digest},
-                stream=body is not None,
-                exceptions_for_status={
-                    403: DatasetWriteError(dataset.id),
-                    404: DatasetNotFound(self._not_found_err_msg(dataset.id)),
-                },
-            )
+            @retriable()
+            def finalize():
+                assert dataset.storage is not None  # for mypy
+                self.request(
+                    f"datasets/{dataset.storage.id}/files/{str(target)}",
+                    method="PUT",
+                    data=body,
+                    token=dataset.storage.token,
+                    base_url=dataset.storage.address,
+                    headers=None if not digest else {self.HEADER_DIGEST: digest},
+                    stream=body is not None,
+                    exceptions_for_status={
+                        403: DatasetWriteError(dataset.id),
+                        404: DatasetNotFound(self._not_found_err_msg(dataset.id)),
+                    },
+                )
+
+            finalize()
 
             return source_file_wrapper.total_read
+        finally:
+            source_file_wrapper.close()
 
     def _iter_files(self, storage: DatasetStorage) -> Generator[FileInfo, None, None]:
         from collections import deque
@@ -661,12 +794,11 @@ class DatasetClient(ServiceClient):
         chunk_size: int = 1024,
         offset: int = 0,
         length: int = -1,
-        max_retries: int = 5,
         validate_checksum: bool = True,
     ) -> Generator[bytes, None, None]:
         import hashlib
 
-        def stream_file(offset: int, length: int) -> Generator[bytes, None, None]:
+        def stream_file() -> Generator[bytes, None, None]:
             headers = {}
             if offset > 0 and length > 0:
                 headers["Range"] = f"bytes={offset}-{offset + length - 1}"
@@ -688,16 +820,18 @@ class DatasetClient(ServiceClient):
         retries = 0
         while True:
             try:
-                for chunk in stream_file(offset, length):
+                for chunk in stream_file():
                     offset += len(chunk)
                     if sha256_hash is not None:
                         sha256_hash.update(chunk)
                     yield chunk
                 break
-            except HTTPError:
-                if retries >= max_retries:
+            except RequestException as err:
+                if retries < self.beaker.MAX_RETRIES:
+                    log_and_wait(retries, err)
+                    retries += 1
+                else:
                     raise
-                retries += 1
 
         # Validate digest.
         if sha256_hash is not None:
@@ -722,15 +856,29 @@ class DatasetClient(ServiceClient):
         total_bytes = 0
         target_dir = target_path.parent
         target_dir.mkdir(exist_ok=True, parents=True)
-        tmp_target = tempfile.NamedTemporaryFile("w+b", dir=target_dir, delete=False, suffix=".tmp")
-        try:
-            for chunk in self._stream_file(storage, file_info, validate_checksum=validate_checksum):
-                total_bytes += len(chunk)
-                tmp_target.write(chunk)
-                progress.update(task_id, advance=len(chunk))
-            os.replace(tmp_target.name, target_path)
-        finally:
-            tmp_target.close()
-            if os.path.exists(tmp_target.name):
-                os.remove(tmp_target.name)
-        return total_bytes
+
+        @retriable(
+            on_failure=lambda: progress.advance(task_id, -total_bytes),
+            recoverable_errors=(RequestException, ChecksumFailedError),
+        )
+        def download() -> int:
+            nonlocal total_bytes
+
+            tmp_target = tempfile.NamedTemporaryFile(
+                "w+b", dir=target_dir, delete=False, suffix=".tmp"
+            )
+            try:
+                for chunk in self._stream_file(
+                    storage, file_info, validate_checksum=validate_checksum
+                ):
+                    total_bytes += len(chunk)
+                    tmp_target.write(chunk)
+                    progress.update(task_id, advance=len(chunk))
+                os.replace(tmp_target.name, target_path)
+            finally:
+                tmp_target.close()
+                if os.path.exists(tmp_target.name):
+                    os.remove(tmp_target.name)
+            return total_bytes
+
+        return download()
