@@ -1,6 +1,16 @@
 import time
 from datetime import datetime, timedelta
-from typing import TYPE_CHECKING, Any, Dict, Generator, List, Sequence, Set, Union
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    Generator,
+    List,
+    Optional,
+    Sequence,
+    Set,
+    Union,
+)
 
 from ..aliases import PathOrStr
 from ..data_model import *
@@ -24,7 +34,8 @@ class ExperimentClient(ServiceClient):
 
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
 
         def _get(id: str) -> Experiment:
@@ -67,7 +78,8 @@ class ExperimentClient(ServiceClient):
         :raises WorkspaceNotSet: If neither ``workspace`` nor
             :data:`Beaker.config.default_workspace <beaker.Config.default_workspace>` are set.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
 
         """
         self.validate_beaker_name(name)
@@ -96,7 +108,8 @@ class ExperimentClient(ServiceClient):
 
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         experiment_id = self.resolve_experiment(experiment).id
         return ExperimentSpec.from_json(
@@ -115,7 +128,8 @@ class ExperimentClient(ServiceClient):
 
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         experiment_id = self.resolve_experiment(experiment).id
         self.request(
@@ -132,7 +146,8 @@ class ExperimentClient(ServiceClient):
 
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         experiment_id = self.resolve_experiment(experiment).id
         self.request(
@@ -150,7 +165,8 @@ class ExperimentClient(ServiceClient):
 
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         experiment = self.resolve_experiment(experiment)
         if delete_results_datasets:
@@ -178,7 +194,8 @@ class ExperimentClient(ServiceClient):
         :raises ValueError: If the new name is invalid.
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         self.validate_beaker_name(name)
         experiment_id = self.resolve_experiment(experiment).id
@@ -202,7 +219,8 @@ class ExperimentClient(ServiceClient):
 
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
 
         :examples:
 
@@ -256,7 +274,8 @@ class ExperimentClient(ServiceClient):
         :raises TaskNotFound: If the given task doesn't exist.
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         exp = self.resolve_experiment(experiment)
         job = self.latest_job(exp, task=task, ensure_finalized=False)
@@ -292,7 +311,8 @@ class ExperimentClient(ServiceClient):
         :raises TaskNotFound: If the given task doesn't exist.
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         exp = self.resolve_experiment(experiment)
         job = self.latest_job(exp, task=task, ensure_finalized=True)
@@ -320,7 +340,8 @@ class ExperimentClient(ServiceClient):
         :raises TaskNotFound: If the given task doesn't exist.
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         exp = self.resolve_experiment(experiment)
         job = self.latest_job(exp, task=task, ensure_finalized=True)
@@ -368,7 +389,8 @@ class ExperimentClient(ServiceClient):
         :raises TaskStoppedError: If ``strict=True`` and a task is stopped
             before a corresponding job is initialized.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         exp_id_to_position: Dict[str, int] = {}
         exps_to_wait_on: List[Experiment] = []
@@ -427,7 +449,8 @@ class ExperimentClient(ServiceClient):
         :raises TaskStoppedError: If ``strict=True`` and a task is stopped
             before a corresponding job is initialized.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         if timeout is not None and timeout <= 0:
             raise ValueError("'timeout' must be a positive number")
@@ -539,22 +562,31 @@ class ExperimentClient(ServiceClient):
                         quiet=quiet,
                         _progress=jobs_progress,
                     ):
-                        finalized_jobs.add(job.id)
-
                         assert job.execution is not None
                         exp_id = job.execution.experiment
+                        task_id = job.execution.task
 
-                        # Ensure job was successful if `strict==True`.
-                        if strict:
-                            job.check()
+                        if job.status.canceled_code in {
+                            CanceledCode.system_preemption,
+                            CanceledCode.user_preemption,
+                        }:
+                            # Job was preempted. Another job will start soon so we just stop
+                            # tracking this one.
+                            exp_to_task_to_job[exp_id][task_id] = None
+                        else:
+                            finalized_jobs.add(job.id)
 
-                        # Update progress display.
-                        experiments_progress.advance(exp_to_progress_task[exp_id])
+                            # Ensure job was successful if `strict==True`.
+                            if strict:
+                                job.check()
 
-                        # Check if corresponding experiment is now finalized.
-                        if experiment_finalized(exp_id) and exp_id in incomplete_exps:
-                            # Experiment has just completed, yield it.
-                            yield complete_experiment(exp_id)
+                            # Update progress display.
+                            experiments_progress.advance(exp_to_progress_task[exp_id])
+
+                            # Check if corresponding experiment is now finalized.
+                            if experiment_finalized(exp_id) and exp_id in incomplete_exps:
+                                # Experiment has just completed, yield it.
+                                yield complete_experiment(exp_id)
                 else:
                     # Wait for `poll_interval` to give Beaker a chance to register jobs.
                     time.sleep(poll_interval)
@@ -626,7 +658,8 @@ class ExperimentClient(ServiceClient):
         :raises TaskStoppedError: If ``strict=True`` and a task is stopped
             before a corresponding job is initialized.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
 
         :examples:
 
@@ -682,7 +715,8 @@ class ExperimentClient(ServiceClient):
         :raises TaskNotFound: If the given task doesn't exist.
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         experiment = self.resolve_experiment(experiment)
         experiment_url = f"{self.config.agent_address}/ex/{self.url_quote(experiment.id)}"
@@ -719,7 +753,8 @@ class ExperimentClient(ServiceClient):
         :raises TaskNotFound: If the given task doesn't exist.
         :raises ExperimentNotFound: If the experiment can't be found.
         :raises BeakerError: Any other :class:`~beaker.exceptions.BeakerError` type that can occur.
-        :raises HTTPError: Any other HTTP exception that can occur.
+        :raises RequestException: Any other exception that can occur when contacting the
+            Beaker server.
         """
         return self._latest_job(
             self._task(experiment, task).jobs, ensure_finalized=ensure_finalized
